import java.io.*;
import java.math.*;
import java.util.*;

public class TestGen {
    int testCount, subtaskCount;
    PrintWriter testsDesc;

    Random random = new Random(787788);
	int MAX_MN = 100_000;
	int MX = 1_000_000_000;
	int MOD = 1_000_000_007;
    
    int[][][] samples1 = new int[][][]{{{1, 1, 1}, {1, 2, 3}, {3, 2, 1}}};
	int[][][] samples2 = new int[][][]{{{1, 1}, {2, 3}, {1, 2}}};

    void write(int[][] a, int[][] b, String folder, String subtask, int points, String comment) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);
        out.println(a.length + " " + a[0].length);
		for (int i = 0; i < a.length; i++) {
		 	for (int j = 0; j < a[i].length; j++) {
		 	 	out.print(a[i][j]);
				if (j + 1 == a[i].length) {
					out.println();
				} else {
				 	out.print(" ");
				}
			}
		}
		out.println(b.length);
        for (int i = 0; i < b.length; i++) {
            out.print(b[i][0] + " " + b[i][1]);
            out.println();
        }
        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=%s", path, folder, subtask, points, comment));
		System.out.println(String.format("Test %d is generated", testCount));
    }


    void genPreliminary() throws IOException {
        for (int i = 0; i < samples1.length; i++) {
            write(samples1[i], samples2[i], "preliminary", "subtask0", 0, String.format("\"sample %d\"", i + 1));
        }
        testCount = 0;
    }

	void genRandom(int n, int m) throws IOException {
		int a[][] = new int[n][m];
		int b[][] = new int[n * m][2];
		
		for (int i = 0; i < n; i++) {
		 	for (int j = 0; j < m; j++) {
		 	 	a[i][j] = random.nextInt(MX);
			}
		}
		for (int i = 0; i < n * m; i++) {
		 	b[i][0] = random.nextInt(n) + 1;
			b[i][1] = random.nextInt(n) + 1;
		}
		
		write(a, b, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"random, n=%d, m=%d\"", n, m));
	}             

	void genZero() throws IOException {   
	 	int n = 2, m = 5;
		int a[][] = new int[n][m];
		int b[][] = new int[2][2];
		
		for (int i = 0; i < m; i++) {
		 	a[0][i] = random.nextInt(MX);
			a[1][i] = (MOD - a[0][i]) % MX;
		}
		b[0][0] = 1;
		b[0][1] = 2;
		b[1][0] = 2;
		b[1][1] = 1;
		write(a, b, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"zero, n=%d, m=%d\"", n, m));
	}

	void genStep(int n, int m) throws IOException {
		int a[][] = new int[n][m];
		int b[][] = new int[n * m][2];
		
		for (int i = 0; i < n; i++) {
		 	for (int j = 0; j < m; j++) {
		 	 	a[i][j] = random.nextInt(MX);
			}
		}
		for (int i = 0; i < n - 1; i++) {
			b[i][0] = i + 2;
			b[i][1] = i + 1;
		}
		for (int i = n - 1; i < n * m; i++) {
		 	b[i][0] = random.nextInt(n) + 1;
			b[i][1] = random.nextInt(n) + 1;
		}
		
		write(a, b, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"steps, n=%d, m=%d\"", n, m));
	}

	void genMax(int k) throws IOException {
		int n = 1, m = 1;
		int a[][] = new int[n][m];
		int b[][] = new int[k][2];
		
		for (int i = 0; i < n; i++) {
		 	for (int j = 0; j < m; j++) {
		 	 	a[i][j] = MX;
			}
		}
		for (int i = 0; i < k; i++) {
			b[i][0] = 1;
			b[i][1] = 1;
		}
		
		write(a, b, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"max, n=%d, m=%d\"", n, m));
	}

    void genSubtask(int mx, int points) throws IOException {
        ++subtaskCount;

		int s = (int)Math.sqrt(mx);
		int o = (int)(1.5 * s);
		genRandom(1, mx);
		genRandom(2, mx / 2);
		genRandom(mx, 1);
		genRandom(mx / 2, 2);
		genRandom(s, s);
		genRandom(s - 1, s + 1);
		genRandom(s + 1, s - 1);
		genRandom(3, mx / 3);
		genRandom(4, mx / 4);
		genRandom(mx / 3, 3);
		genRandom(mx / 4, 4);
		genRandom(o, mx / o);
		genRandom(mx / o, o);
		genZero();
		genStep(o, mx / o);
		genStep(mx / o, o);
		genMax(mx);
        
        testsDesc.println(String.format("comment_subtask%d=\"n*m &lt;= %d\"", subtaskCount, mx));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));       
    }

    void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genPreliminary();
        genSubtask(100, 30);
        genSubtask(MAX_MN, 70);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
